###############################################################################
# (c) Copyright 2020-2023 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
macro(_init_from_env)
    file(TO_CMAKE_PATH "$ENV{PATH}" path)
    file(TO_CMAKE_PATH "$ENV{LD_LIBRARY_PATH}" ld_library_path)
    file(TO_CMAKE_PATH "$ENV{PYTHONPATH}" pythonpath)
    file(TO_CMAKE_PATH "$ENV{ROOT_INCLUDE_PATH}" root_include_path)
    file(TO_CMAKE_PATH "$ENV{PKG_CONFIG_PATH}" pkg_config_path)
    set(pythonpath) # empty the PYTHONPATH
endmacro()

macro(_add_lcg_entry entry)
    list(APPEND CMAKE_PREFIX_PATH ${entry})
    if(EXISTS ${entry}/bin)
        list(PREPEND path ${entry}/bin)
    endif()
    foreach(_suffix IN ITEMS lib lib64)
        if(EXISTS ${entry}/${_suffix})
            file(GLOB inner_files "${entry}/${_suffix}/*.so*")
            if(inner_files)
                list(PREPEND ld_library_path ${entry}/${_suffix})
            endif()
            file(GLOB inner_files "${entry}/${_suffix}/*.py")
            if(inner_files)
                list(PREPEND pythonpath ${entry}/${_suffix})
            endif()
            if(EXISTS "${entry}/${_suffix}/python${_python_version}/site-packages")
                list(PREPEND pythonpath ${entry}/${_suffix}/python${_python_version}/site-packages)
            endif()
        endif()
    endforeach()
    if(EXISTS ${entry}/include)
        list(PREPEND root_include_path ${entry}/include)
    endif()
    # ${entry}/lib/pkgconfig and ${entry}/lib64/pkgconfig may not be automatically checked
    foreach(_pkg_lib IN ITEMS lib lib64)
        if(EXISTS ${entry}/${_pkg_lib}/pkgconfig)
            list(PREPEND pkg_config_path ${entry}/${_pkg_lib}/pkgconfig)
        endif()
    endforeach()
endmacro()

macro(_add_lbenv_workspace platform)
    if(IS_DIRECTORY "$ENV{LBENV_CURRENT_WORKSPACE}")
        file(GLOB workspace_dirs "$ENV{LBENV_CURRENT_WORKSPACE}/*/InstallArea/${platform}")
        list(PREPEND CMAKE_PREFIX_PATH ${workspace_dirs})
    endif()
endmacro()

macro(_update_env)
    foreach(_lst IN ITEMS path ld_library_path pythonpath root_include_path pkg_config_path)
        list(REMOVE_DUPLICATES ${_lst})
    endforeach()
    # file(TO_NATIVE_PATH) does not transform ; to :
    string(REPLACE ";" ":" path "${path}")
    set(ENV{PATH} "${path}")
    string(REPLACE ";" ":" ld_library_path "${ld_library_path}")
    set(ENV{LD_LIBRARY_PATH} "${ld_library_path}")
    string(REPLACE ";" ":" pythonpath "${pythonpath}")
    set(ENV{PYTHONPATH} "${pythonpath}")
    string(REPLACE ";" ":" root_include_path "${root_include_path}")
    set(ENV{ROOT_INCLUDE_PATH} "${root_include_path}")
    string(REPLACE ";" ":" pkg_config_path "${pkg_config_path}")
    set(ENV{PKG_CONFIG_PATH} "${pkg_config_path}")
endmacro()

macro(_fix_pkgconfig_search)
    # tuning of cmake delegation to pkg-config (https://cmake.org/cmake/help/latest/module/FindPkgConfig.html)
    # if we let FindPkgConfig rely on CMAKE_PREFIX_PATH we will miss either lib or lib64 subdirs
    # - make sure pkg-config uses CMAKE_PREFIX_PATH
    set(PKG_CONFIG_USE_CMAKE_PREFIX_PATH YES)
    # - when crosscompiling, CMake ignores /usr/lib64/pkgconfig, but this is kind of a hybrid
    if(EXISTS /usr/lib64/pkgconfig AND NOT "$ENV{PKG_CONFIG_PATH}" MATCHES ":/usr/lib64/pkgconfig")
        set(ENV{PKG_CONFIG_PATH} "$ENV{PKG_CONFIG_PATH}:/usr/lib64/pkgconfig")
    endif()
endmacro()

macro(_set_pythonhome pythonhome)
    # PYTHONHOME needed by gdb
    set(ENV{PYTHONHOME} "${pythonhome}")

    # prevent find_package(Python) too look into the path
    set(Python_ROOT_DIR "${pythonhome}")
    set(Python2_ROOT_DIR "${pythonhome}")
    set(Python3_ROOT_DIR "${pythonhome}")

    # we make sure we set the CMake python lookup policy to use the first
    # one found in the path (i.e. from LCG)
    set(Python_FIND_STRATEGY LOCATION)
endmacro()
